/*
    This file is part of KolabAdmin.

    Copyright (C) 2006 Tobias Koenig <tobias.koenig@credativ.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "connection.h"
#include "tool.h"

#include "administrator_record.h"

using namespace Form;

Entry AdministratorRecord::loadEntry( const QString &id, PagePolicy::State )
{
  QString filter( "objectClass=*" );
  QLdapResponse response = Connection::self()->search( id, QLdap::Base, filter );

  if ( !response.isValid() || response.entries().isEmpty() )
    return Entry();

  return Entry::fromLdapEntry( response.entries().first() );
}

bool AdministratorRecord::saveEntry( const Entry &entry, PagePolicy::State state, QString &errorMsg )
{
  if ( state == PagePolicy::Add || state == PagePolicy::Modify ) {

    QString dnRoot = QString( "cn=internal,%1" ).arg( Connection::self()->baseDn() );

    QLdapEntry ldapEntry;

    ldapEntry.addValue( "objectClass", "top" );
    ldapEntry.addValue( "objectClass", "inetOrgPerson" );
    ldapEntry.addValue( "objectClass", "kolabInetOrgPerson" );

    ldapEntry.setValue( "givenName", entry.value( "givenName" ) );
    ldapEntry.setValue( "sn", entry.value( "sn" ) );
    ldapEntry.setValue( "cn", QString( "%1 %2" ).arg( entry.value( "givenName" ), entry.value( "sn" ) ) );

    if ( !entry.value( "password" ).isEmpty() ) {
      QString data = QString( "{CRYPT}%1" )
            .arg( QString::fromLocal8Bit( crypt( entry.value( "password" ).toLocal8Bit().data(), "tK" ) ) );
      ldapEntry.setValue( "userPassword", data );
    } else {
      ldapEntry.setValue( "userPassword", entry.value( "userPassword" ) );
    }

    ldapEntry.setValue( "uid", entry.value( "uid" ).toLower() );

    if ( state == PagePolicy::Modify ) {
      QString newDn = QString( "cn=%1,%2" ).arg( ldapEntry.value( "cn" ), dnRoot );

      /**
       * The 'cn' of the administrator has changed, so we have to perform some
       * extra checks.
       */
      if ( entry.id() != newDn ) {
        /**
         * Add the object under the new dn...
         */
        if ( !Connection::self()->add( newDn, ldapEntry ) ) {
          errorMsg = QObject::tr( "Could not modify administrator '%1': %2" )
                                .arg( ldapEntry.value( "cn" ), Connection::self()->errorString() );
          return false;
        }

        /**
         * ... and remove the object under the old dn.
         */
        if ( !Connection::self()->remove( entry.id() ) ) {
          errorMsg = QObject::tr( "Could not modify administrator '%1': %2" )
                                .arg( ldapEntry.value( "cn" ), Connection::self()->errorString() );
          return false;
        }

        /**
         * Add the new administrator dn to the admin group...
         */
        QString adminGroupDn = QString( "cn=admin,%1" ).arg( dnRoot );
        QLdapEntry memberEntry;

        memberEntry.setValue( "member", newDn );
        if ( !Connection::self()->addAttributes( adminGroupDn, memberEntry ) ) {
          errorMsg = QObject::tr( "Could not modify administrator '%1': %2" )
                                .arg( ldapEntry.value( "cn" ), Connection::self()->errorString() );
          return false;
        }

        /**
         * ... and remove the old administrator dn from the admin group.
         */
        memberEntry.setValue( "member", entry.id() );
        if ( !Connection::self()->removeAttributes( adminGroupDn, memberEntry ) ) {
          errorMsg = QObject::tr( "Could not modify administrator '%1': %2" )
                                .arg( ldapEntry.value( "cn" ), Connection::self()->errorString() );
          return false;
        }
      } else {
        if ( !Connection::self()->modifyAttributes( entry.id(), ldapEntry ) ) {
          errorMsg = QObject::tr( "Could not modify administrator '%1': %2" )
                                .arg( ldapEntry.value( "cn" ), Connection::self()->errorString() );
          return false;
        }
      }
    } else if ( state == PagePolicy::Add ) {

      QString dn = QString( "cn=%1,%2" ).arg( ldapEntry.value( "cn" ), dnRoot );

      /**
       * First add the administrator...
       */
      if ( !Connection::self()->add( dn, ldapEntry ) ) {
        errorMsg = QObject::tr( "Could not add administrator '%1': %2" )
                              .arg( ldapEntry.value( "cn" ), Connection::self()->errorString() );
        return false;
      }

      /**
       * ... then add the administrator dn to the admin group.
       */
      QLdapEntry memberEntry;
      memberEntry.setDn( QString( "cn=admin,cn=internal,%1" ).arg( Connection::self()->baseDn() ) );
      memberEntry.setValue( "member", dn );
      if ( !Connection::self()->addAttributes( memberEntry.dn(), memberEntry ) ) {
        errorMsg = QObject::tr( "Could not add administrator '%1' to group '%2': %3" )
                      .arg( entry.value( "cn" ), memberEntry.dn(), Connection::self()->errorString() );
        return false;
      }
    }
  } else if ( state == PagePolicy::Delete ) {
    /**
     * First remove administrator dn from admin group...
     */
    QLdapEntry memberEntry;
    memberEntry.setDn( QString( "cn=admin,cn=internal,%1" ).arg( Connection::self()->baseDn() ) );
    memberEntry.setValue( "member", entry.id() );
    if ( !Connection::self()->removeAttributes( memberEntry.dn(), memberEntry ) ) {
      errorMsg = QObject::tr( "Could not remove administrator '%1' from group '%2': %3" )
                    .arg( entry.value( "cn" ), memberEntry.dn(), Connection::self()->errorString() );
      return false;
    }

    /**
     * ... then remove the administrator itself.
     */
    QString ldapError;
    if ( !Tool::deleteObject( Connection::self(), entry.id(), ldapError ) ) {
      errorMsg = QObject::tr( "Could not remove administrator '%1': %2" )
                    .arg( entry.value( "cn" ), ldapError );
      return false;
    }
  }

  return true;
}
