/*
    This file is part of KolabAdmin.

    Copyright (C) 2006 Tobias Koenig <tobias.koenig@credativ.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>
#include <QMessageBox>
#include <QPushButton>
#include <QSettings>
#include <QSpinBox>

#include "auth.h"
#include "connection.h"

#include "connectiondialog.h"

class ConfigureDialog : public QDialog
{
  public:
    ConfigureDialog( QWidget *parent = 0 );

  public Q_SLOTS:
    virtual void accept();

  private:
    QLineEdit *mHost;
    QSpinBox *mPort;
    QLineEdit *mBaseDn;
    QLineEdit *mUser;
    QLineEdit *mPassword;
};

ConnectionDialog::ConnectionDialog( QWidget *parent )
  : QDialog( parent )
{
  QGridLayout *layout = new QGridLayout( this );

  QLabel *label = new QLabel( this );
  label->setPixmap( QPixmap( ":kolab_logo" ) );
  layout->addWidget( label, 0, 0 );

  label = new QLabel( tr( "User:" ), this );
  mUser = new QLineEdit( this );
  label->setBuddy( mUser );

  layout->addWidget( label, 1, 0 );
  layout->addWidget( mUser, 1, 1, 1, 4 );

  label = new QLabel( tr( "Password:" ), this );
  mPassword = new QLineEdit( this );
  mPassword->setEchoMode( QLineEdit::Password );
  label->setBuddy( mPassword );

  layout->addWidget( label, 2, 0 );
  layout->addWidget( mPassword, 2, 1, 1, 4 );

  QPushButton *button = new QPushButton( tr( "Quit" ), this );
  connect( button, SIGNAL( clicked() ), this, SLOT( reject() ) );
  layout->addWidget( button, 3, 1 );

  button = new QPushButton( tr( "Configure ..." ), this );
  connect( button, SIGNAL( clicked() ), this, SLOT( configure() ) );
  layout->addWidget( button, 3, 2 );

  layout->addItem( new QSpacerItem( 70, 20 ), 3, 3 );

  mOkButton = new QPushButton( tr( "Ok" ), this );
  mOkButton->setDefault( true );
  connect( mOkButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
  layout->addWidget( mOkButton, 3, 4 );

  // tokoe: debug
  QSettings settings( "Kolab", "KolabAdmin" );
  settings.beginGroup( "session" );
  mUser->setText( settings.value( "user" ).toString() );
  mPassword->setText( settings.value( "password" ).toString() );
  settings.endGroup();
  // tokoe: end debug

  updateButtonState();
}

void ConnectionDialog::accept()
{
  if ( mUser->text().isEmpty() || mPassword->text().isEmpty() ) {
    QMessageBox::critical( 0, QString(), tr( "You have to enter the login data!" ) );
    return;
  }

  QSettings settings( "Kolab", "KolabAdmin" );
  settings.beginGroup( "connection" );
  const QString host = settings.value( "host" ).toString();
  const int port = settings.value( "port", QVariant( 389 ) ).toInt();
  const QString baseDn = settings.value( "basedn" ).toString();
  const QString user = settings.value( "nobody_user" ).toString();
  const QString password = settings.value( "nobody_password" ).toString();
  settings.endGroup();

  QLdap ldap( this );
  ldap.setHost( host );
  ldap.setPort( port );
  ldap.setBaseDn( baseDn );
  ldap.setUser( user );
  ldap.setPassword( password );

  const QString filter = QString( "(&(objectClass=kolabInetOrgPerson)(|(uid=%1)(mail=%2)))" )
                                .arg( mUser->text() ).arg( mUser->text() );
  QLdapResponse response = ldap.search( baseDn, QLdap::Sub, filter, QStringList( "dn" ) );
  if ( !response.isValid() ) {
    QMessageBox::critical( this, QString(), tr( "Unable to resolve login to dn: %1" ).arg( ldap.errorString() ) );
    return;
  } else if ( response.entries().isEmpty() ) {
    QMessageBox::critical( this, QString(), tr( "User '%1' is unknown to the system." ).arg( mUser->text() ) );
    return;
  }

  Connection::self()->setHost( host );
  Connection::self()->setPort( port );
  Connection::self()->setBaseDn( baseDn );
  Connection::self()->setUser( response.entries().first().dn() );
  Connection::self()->setPassword( mPassword->text() );

  QString errorMsg;

  if ( !Auth::self()->authenticate( response.entries().first().dn(), errorMsg ) ) {
    QMessageBox::critical( 0, QString(), errorMsg );
    return;
  }

  // tokoe: debug
  settings.beginGroup( "session" );
  settings.setValue( "user", mUser->text() );
  settings.setValue( "password", mPassword->text() );
  settings.endGroup();
  settings.sync();
  // tokoe: end debug

  done( Accepted );
}

void ConnectionDialog::configure()
{
  ConfigureDialog dlg( this );

  if ( dlg.exec() )
    updateButtonState();
}

void ConnectionDialog::updateButtonState()
{
  QSettings settings( "Kolab", "KolabAdmin" );
  if ( settings.contains( "connection/host" ) )
    mOkButton->setEnabled( true );
  else
    mOkButton->setEnabled( false );
}

ConfigureDialog::ConfigureDialog( QWidget *parent )
  : QDialog( parent )
{
  QGridLayout *layout = new QGridLayout( this );

  QLabel *label = new QLabel( tr( "Host:" ), this );
  mHost = new QLineEdit( this );
  label->setBuddy( mHost );

  layout->addWidget( label, 0, 0 );
  layout->addWidget( mHost, 0, 1, 1, 2 );

  label = new QLabel( tr( "Port:" ), this );
  mPort = new QSpinBox( this );
  mPort->setRange( 1, 65536 );
  mPort->setValue( 389 );
  label->setBuddy( mPort );

  layout->addWidget( label, 1, 0 );
  layout->addWidget( mPort, 1, 1, 1, 2 );

  label = new QLabel( tr( "Base Dn:" ), this );
  mBaseDn = new QLineEdit( this );
  label->setBuddy( mBaseDn );

  layout->addWidget( label, 2, 0 );
  layout->addWidget( mBaseDn, 2, 1, 1, 2 );

  label = new QLabel( tr( "Nobody User:" ), this );
  mUser = new QLineEdit( this );
  label->setBuddy( mUser );

  layout->addWidget( label, 3, 0 );
  layout->addWidget( mUser, 3, 1, 1, 2 );

  label = new QLabel( tr( "Nobody Password:" ), this );
  mPassword = new QLineEdit( this );
  mPassword->setEchoMode( QLineEdit::Password );
  label->setBuddy( mPassword );

  layout->addWidget( label, 4, 0 );
  layout->addWidget( mPassword, 4, 1, 1, 2 );

  QPushButton *button = new QPushButton( tr( "Ok" ), this );
  connect( button, SIGNAL( clicked() ), this, SLOT( accept() ) );
  layout->addWidget( button, 5, 1 );

  button = new QPushButton( tr( "Cancel" ), this );
  connect( button, SIGNAL( clicked() ), this, SLOT( reject() ) );
  layout->addWidget( button, 5, 2 );

  QSettings settings( "Kolab", "KolabAdmin" );
  settings.beginGroup( "connection" );
  mHost->setText( settings.value( "host" ).toString() );
  mPort->setValue( settings.value( "port", QVariant( 389 ) ).toInt() );
  mBaseDn->setText( settings.value( "basedn" ).toString() );
  mUser->setText( settings.value( "nobody_user" ).toString() );
  mPassword->setText( settings.value( "nobody_password" ).toString() );
}

void ConfigureDialog::accept()
{
  if ( mHost->text().isEmpty() || mBaseDn->text().isEmpty() ||
       mUser->text().isEmpty() || mPassword->text().isEmpty() ) {
    QMessageBox::critical( this, tr( "Invalid Input" ), tr( "You have to fill out all fields" ) );
    return;
  }

  QSettings settings( "Kolab", "KolabAdmin" );
  settings.beginGroup( "connection" );
  settings.setValue( "host", mHost->text() );
  settings.setValue( "port", mPort->value() );
  settings.setValue( "basedn", mBaseDn->text() );
  settings.setValue( "nobody_user", mUser->text() );
  settings.setValue( "nobody_password", mPassword->text() );
  settings.endGroup();
  settings.sync();

  done( Accepted );
}
